//+------------------------------------------------------------------+
//|                                        TheHolyMackerel_v2.0.0.mq4 |
//+------------------------------------------------------------------+


// this is a customized version of HG v1.6
// fixed lot size option
// cleaned up code
// changed RSI "active" vs "inactive" -- I like slowing things down to RSI=4 for H1, M15 in inactive hours
// more strict checking on open orders by both ticket and position,
//   immediately after opening an order OrderSelect() by MODE_TRADES can fail to
//   return recently added order, so multiple orders were placed.
//   This code requires both a check by ticket and by open trades.
// writes informational messages regarding trends to log every 5 minutes.
// uses LibGMT.mqh for time calcs
// uses LibOrderReliable.mqh for orders.  Supports ECN.
// corrected loop over open orders: was previously <= OrdersTotal(), should be < OrdersTotal()

/* updates for V1.7

*) Trend determination has new options
1) original 4 TF scooby
2) modification of #1 excluding dailiy
3) EMA(25) > EMA(50) > EMA(100) (1H) type of trend

*) Uses LaguerreRSI for actual entry instead of M5 RSI

*) TakeProfit/Stop Loss default set back to Scooby THG 1.6 Original (smaller take profit, higher win rate, larger stop), made
made configurable.

Still not yet implemented: news filter.

Updates for V1.71

*) dynamic lot size computation

Updates for V1.72

*) Fix handling/bugs with high position count

Updates for V1.73

*) Improve trend logging to journal.
*) Code cleanup/simplification

Updates for V1.80

*) change to stop positioning under swing low.

Updates for V1.90

*) user-selectable stop positioning under swing low or ATR
*) user-selectable lot size positioning by TP or by SL
*) reduction of lot size with increasing number of open positions.

Updates for V1.91

*) actually fix bugs to implement 1.90

Updates for V1.92

*) implement 'RSI Quality' statistic to check whether sum of open-close of bars is a certain fraction of
   net high-low distnace.

*/

/*
  updates for v2.0.0

  * Convert to LibOrderReliable4
  * Change TF for LaguerreRSI to M15 from M05
  * Change default ATR from M15 to
*/

/* updates for v2.0.1
    * check minimum lot volume upon making trade

   updates for v2.0.2
    * emit debug output for triggers
    * clear triggers if trade already found

   updates for v2.0.3
    * backtest works (single symbol)
    * LagRSI TF is changable
 */



#property show_inputs
#include <LibOrderReliable4.mqh>
#include <LibGMT.mqh>


//+------------------------------------------------------------------+
//| expert External variables                                        |
//+------------------------------------------------------------------+

extern int Magic = 1603;
string Key = "TheHolyMackerel v2.0.3";

extern int OrderLimit = 6;
extern bool LotFixed = false;
extern double fixedLot = 0.1;
extern double riskPct = 1.0;
extern bool LotSizeFromTPnotSL=false;
//extern string msg1="Set the next param to TRUE for ECNs which require two step orders.";
extern bool UseTwoStepOrders = true; // true is default since it is usually safe to use on non ECN
extern int TrendMethod=2; // 1 == 4 TF 20/200 EMA, 2 is 3 TF 20/200 EMA, 3 is EMA(25)/EMA(50)/EMA(100) on H1
extern int TPSL_ATR_period= 24;
extern int TPSL_ATR_TF    = 60;
extern string msg="Set the next to true to set by swing extreme, probably want smaller SL_multiplier";
extern bool SetSLfromSwingExtreme = false;
extern double TP_multiplier = 1.0;
extern double SL_multiplier = 2.0;
extern int    LaguerreRSI_TF=15;
extern double LaguerreRSIgamma=0.5;
extern double LaguerreRSIentryCriterion = 0.10;
extern double LaguerreRSIfireCriterion  = 0.15;
extern double H4_OverSoldPct = 20.0;  // will compute OverBought as 100-OverSold
extern double H1_OverSoldPct = 20.0;
extern double M15_OverSoldPct = 20.0;
extern double Min_RSI_Quality = 45.0;
extern string depositCurrency = "USD";

/*
  New 1.7 LaguerreRSI logic.

  To set up a buy, we must have the MTF RSI's in over-sold and  Laguerre RSI < LaguerreRSIentryCriterion.
  That will 'cock' a buy trigger.  As soon as  Laguerre RSI > LaguerreRSIexitCriterion, trigger pulled.
  and a buy order will be sent.  Of course reverse for sells.

  How to uncock a trade:  (a) enter a trade  (b) trend is no longer in right direction
  (c) trading time is expired.   Future, once news filter is in place, news time?
*/



//+------------------------------------------------------------------+
//| expert Internal variables                                        |
//+------------------------------------------------------------------+

//string Pairs[] = {"AUDJPY","AUDUSD","CHFJPY","EURGBP","EURJPY","EURUSD",
//                  "GBPJPY","GBPUSD","USDCAD","USDCHF","USDJPY"};

string Pairs[] = { "AUDJPY", "AUDUSD", "CADJPY", "CHFJPY", "EURAUD", "EURCAD", "EURGBP",
                   "EURJPY", "EURUSD", "GBPJPY", "GBPUSD", "USDCAD", "USDCHF", "USDJPY"
                 };

/* tickets for actually open trades */
int PairTickets[] = { 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0
                    };

/* triggered to +1/-1 for buy/sell to set up trade
   once laguerreRSI on 5M is recovered */

int PairTriggers[] = { 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0
                     };


string Author = "Matt Kennel 2010.";

int M05 = PERIOD_M5;
int M15 = PERIOD_M15;
int M30 = PERIOD_M30;
int H01 = PERIOD_H1;
int H04 = PERIOD_H4;
int D01 = PERIOD_D1;


int Npairs;

//+------------------------------------------------------------------+
//| expert Initialization function                                   |
//+------------------------------------------------------------------+

int init()
{
    string Suffix = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);

    if ( IsTesting() == true) {
        Npairs = 1;
        Pairs[0] = Symbol();
    } else {
        Npairs = ArraySize(Pairs);
        for (int a = 0; a < Npairs; a++) {
            Pairs[a] = Pairs[a] + Suffix;
        }
        for (a = 0; a < Npairs; a++) {
            int ticket = TicketOfExistingTrade(Pairs[a], Magic);
            PairTickets[a] = ticket;
            if (ticket > 0) {
                Print("init(): found existing open position ", Pairs[a], "/", ticket);
            }
        }
    }
    if (Magic == 0) {
        Magic = AccountNumber();
    }
    Comment("\nWaiting for tick update...");
    O_R_Config_use2step(UseTwoStepOrders);
    return (0);
}

//+------------------------------------------------------------------+
//| expert Deinitialization function                                 |
//+------------------------------------------------------------------+

int deinit()
{
    Comment("");
    return (0);
}

//+------------------------------------------------------------------+
//| expert Start function                                            |
//+------------------------------------------------------------------+


int start()
{
    string Status = "Trading enabled...";

    int gmtHHMM, gmtday;
    GetGMTDayHHMM(gmtday, gmtHHMM);

    if ((gmtday == 5 && gmtHHMM >= 1200) ||
            (gmtday == 0 && gmtHHMM < 2300) || gmtday == 6) {
        // do not trade on Friday past GMT 12:00, do not trade sunday before AUD/JPY open, or Saturday ever,
        // if you want add news filter
        Comment("Trading disabled for time blockout...");
        for (int i=0; i < Npairs; i++) {
            PairTriggers[i] = 0;
        }
        return(0);
    }

    string commentstr="\n" +  Author + " "+ Status+"\n";
    string trendmsg="";
    static string prevtrendmsg="";
    int ntrending = 0;

    static datetime lastTrendMsgTime = 0;
    static datetime lastTickTime=0;
    if (TimeCurrent() == lastTickTime) {
        return(0);
        /* only bother to update once per second, to reduce CPU load during high tick volume */
    } else {
        lastTickTime=TimeCurrent();
    }

    for (int b = 0; b < Npairs; b++) {
        string thissym = Pairs[b];
        bool priceOK = MarketInfo(thissym, MODE_BID) > 0 && MarketInfo(thissym, MODE_ASK) > 0;

        if (!priceOK) {
            Print("PriceOK failed for: ", thissym);
        }

        if (priceOK) {
            double mylots = VolAndBalanceAdjustedLotsize(thissym);
            //           Print("Price OK succeeded for: ",thissym);
            int RsiH4 = 3;
            int RsiH1 = 3;
            int RsiM15 = 3;
            int RsiM05 = 3;
            bool IsActive = IsPairActive(thissym, gmtHHMM);
            if (!IsActive) {
                RsiH1 = 4;  /* slow down time during inactive hours */
                RsiM15 = 4;
            }

            double RSI_M15 = iRSI(thissym, M15, RsiM15, PRICE_CLOSE, 0);
            double RSI_H01 = iRSI(thissym, H01, RsiH1, PRICE_CLOSE, 0);
            double RSI_H04 = iRSI(thissym, H04, RsiH4, PRICE_CLOSE, 0);

            double RSIQ_M15 = RsiQuality(thissym,M15,RsiM15,0);
            double RSIQ_H01 = RsiQuality(thissym,H01,RsiH1,0);
            double RSIQ_H04 = RsiQuality(thissym,H04,RsiH4,0);


            commentstr = commentstr + thissym+"/"+DoubleToStr(mylots,2)+
                         " H4:"+DoubleToStr(RSI_H04,1)+" ("+DoubleToStr(RSIQ_H04,0)+") "+
                         " H1:"+DoubleToStr(RSI_H01,1)+" ("+DoubleToStr(RSIQ_H01,0)+") "+
                         " M15:"+DoubleToStr(RSI_M15,1)+" ("+DoubleToStr(RSIQ_M15,0)+")\n";


            string trendstr="none";
            int trend = Trend(thissym,trendstr);

            if (PairTriggers[b] != 0) {
                /* handle cancellation of cocked trade due to trend change */
                if (PairTriggers[b] != trend) {
                    Print(thissym, "@",TimeToStr(TimeCurrent())," ", buySellFlatString(PairTriggers[b]), " TRIGGER CANCELLED FOR TREND CHANGE." );
                    PairTriggers[b] = 0;
                }
                /* handle unexpected condition of open order AND cocked trigger */
                if ((TradeExistTicker(PairTickets[b]) == true) || TradeExistsInOpenOrders(thissym,Magic)) {
                    Print(thissym, "@",TimeToStr(TimeCurrent())," ", buySellFlatString(PairTriggers[b]), " TRIGGER CANCELLED FOR EXISTING OPEN TRADE. WTF?" );
                    PairTriggers[b] = 0;
                }
            }

            /* it is possible that a trade has just closed, so clear the PairTickets[] for it */
            if (TradeExistTicker(PairTickets[b]) == false) {
                PairTickets[b] = 0;
            }

            /* add to trend message */
            if (trend != 0) {
                ntrending++;
                trendmsg = trendmsg + thissym + ":" + trendstr;
            } else {
                trendmsg = trendmsg + lc(thissym)+":" + trendstr;
            }
            if (IsActive) {
                trendmsg = trendmsg + "a ";
            } else {
                trendmsg = trendmsg + "i ";
            }
            if (PairTriggers[b] > 0) {
                trendmsg = trendmsg + ",cocked BUY";
            }
            if (PairTriggers[b] < 0) {
                trendmsg = trendmsg + ",cocked SELL";
            }

            // Print("DEBUG: trend for ",thissym," = ",trendmsg);

            double LagRsiNow = LaguerreRSI(thissym,LaguerreRSI_TF, LaguerreRSIgamma, 0);
            //Print("DEBUG: LagRsiNow("+thissym+")=",LagRsiNow);
            bool IsOverSold3TF = OverSold(RSI_M15,M15_OverSoldPct)
                                 && OverSold(RSI_H01,H1_OverSoldPct)
                                 && OverSold(RSI_H04,H4_OverSoldPct);
            bool IsOverBought3TF = OverBought(RSI_M15,M15_OverSoldPct)
                                   && OverBought(RSI_H01,H1_OverSoldPct)
                                   && OverBought(RSI_H04,H4_OverSoldPct);

            bool RSIQ_OK_3TF = RSIQ_OK(RSIQ_M15) && RSIQ_OK(RSIQ_H01) && RSIQ_OK(RSIQ_H04);

            /* if (IsOverSold3TF && (trend==+1)) {
               Print("DEBUG: Potential LONG Entry: ", thissym, " with RSIQ M15/H01/H04=", RSIQ_M15,"/",RSIQ_H01,"/",RSIQ_H04," overall=",RSIQ_OK_3TF);
            }
            if (IsOverBought3TF && (trend==-1)) {
               Print("DEBUG: Potential SHORT Entry: ", thissym, " with RSIQ M15/H01/H04=", RSIQ_M15,"/",RSIQ_H01,"/",RSIQ_H04," overall=",RSIQ_OK_3TF);
            }
            */

            if (PairTriggers[b] == 0) {
                /* check if trade should be prepared for potential trade */
                int cockedTrade=0;
                if ((trend == +1) && IsOverSold3TF && RSIQ_OK_3TF && (LagRsiNow < LaguerreRSIentryCriterion) ) {
                    if (AbleToTrade(thissym, PairTickets[b])) {
                        cockedTrade=+1;
                    }
                }
                if ((trend == -1) && IsOverBought3TF && RSIQ_OK_3TF && (LagRsiNow > (1.0-LaguerreRSIentryCriterion))) {
                    if (AbleToTrade(thissym, PairTickets[b])) {
                        cockedTrade=-1;
                    }
                }
                if (cockedTrade != 0) {
                    PairTriggers[b] = cockedTrade;
                    Print(thissym, "@",TimeToStr(TimeCurrent())," ", buySellFlatString(cockedTrade)," TRIGGER@", " bid=",MarketInfo(thissym,MODE_BID),
                          " ask=",MarketInfo(thissym,MODE_ASK), "  RSI H4/H1/M15=", RSI_H04, "/", RSI_H01, "/", RSI_M15, " LagRSI=",LagRsiNow);
                }
            } else {
                /* pair trigger is set! so check if we need to send and order  */

                if ((PairTriggers[b] == +1) && (LagRsiNow > LaguerreRSIfireCriterion) && (LagRsiNow < 0.5) && AbleToTrade(thissym, PairTickets[b])) {
                    Print("DEBUG: sending order with LagRsiNow("+thissym+")="+LagRsiNow);
                    SendTriggeredOrder(b, OP_BUY, thissym, mylots);
                }
                if ((PairTriggers[b] == -1) && (LagRsiNow < (1.0-LaguerreRSIfireCriterion)) && (LagRsiNow > 0.5) && AbleToTrade(thissym, PairTickets[b])) {
                    Print("DEBUG: sending order with LagRsiNow("+thissym+")="+LagRsiNow);
                    SendTriggeredOrder(b, OP_SELL, thissym, mylots);
                }
            } /* PairTriggers[b] == / != 0 */
        } /* end if PriceOK */
    }   /* master loop over array pairs */
    Comment(commentstr);

    if ((trendmsg != prevtrendmsg) && !IsTesting() ) {
        Print("ntrending=",ntrending, trendmsg);
    }
    prevtrendmsg = trendmsg;
    return (0);
}

void SendTriggeredOrder(int b, int ordertype, string thissym, double mylots)
{
    /*
     * b         -- index into the PairTickets[] and PairTriggers[] arrays.
     * ordertype -- OP_BUY or OP_SELL
     * thissym   -- corresponding Symbol
     * mylots    -- number of lots
     *
     * Send an order which just triggered, handling clearing of
     * existing triggers, setting PairTickets and dealing with
     * issue of too many open positions (i.e. abort attempted trade, clear trigger).
     * The reason we abort instead of hold is that if the position count is too high
     * and we wait until position count goes back down below the limit, the PRICE_CLOSE
     * could have moved far from the desired entry point, resulting in a lower probability
     * trade success. So it is safer to abort.  However, aborting because of position
     * count upon setting the trigger might result in too many missed trade opportunities
     * because numerous triggered trades are canceled because of trend-changes before
     * the trigger condition hits */

    if (PosCount(Magic) >= OrderLimit) {
        Print(thissym, " TRIGGERED BUT PosCount >= OrderLimit SO TRADE ABORTED @", TimeToStr(TimeCurrent())," ",MarketInfo(thissym,MODE_BID));
        PairTickets[b] = 0;
        PairTriggers[b] = 0;
    } else {
        if (IsSpreadOK(thissym,10)) {
            int ticket = SendOrder(thissym, ordertype);
            Print(thissym,"@",TimeToStr(TimeCurrent())," SendOrder returned ticket=",ticket);
  
            if (ticket > 0) {
                PairTickets[b] = ticket;
            } else {
                PairTickets[b] = 0;
                Print(thissym,"@",TimeToStr(TimeCurrent())," SendOrder failed, therefore clearing cocked trigger. :(");
            }
            PairTriggers[b] = 0;  // this is for safety
        }
    }
}

bool AbleToTrade(string symbol, int ticker)
{
    /* this is a computationally expensive function as it has to hit the server.
       use short-circuit if starting with less expensive (check by ticker) first. */
    if ( TradeExistTicker(ticker) || TradeExistsInOpenOrders(symbol,Magic) || !IsTradeAllowed() ) {
        return(false);
    } else {
        return(true);
    }
}

bool PosCountMaxedOut()
{
    return(PosCount(Magic) >= OrderLimit);
}

//+------------------------------------------------------------------+
//| expert positions counter function                                            |
//+------------------------------------------------------------------+

int PosCount(int magic)
{
    int poscnt = 0;

    for (int c = OrdersTotal()-1; c >= 0; c--) {
        if (OrderSelect(c, SELECT_BY_POS, MODE_TRADES) == true) {
            if (OrderMagicNumber() == magic) {
                poscnt++;
            }
        }
    }
    return (poscnt);
}

//+------------------------------------------------------------------+
//| expert TradeExists function                                      |
//+------------------------------------------------------------------+
bool TradeExistsInOpenOrders(string symbol, int magic)
{
    bool Found = false;

    for (int c = OrdersTotal()-1; c >= 0; c--) {
        if (OrderSelect(c, SELECT_BY_POS, MODE_TRADES) == true) {
            if (OrderSymbol() == symbol && OrderMagicNumber() == magic) {
                Found = true;
                break;
            }
        }
    }
    return (Found);
}

/* returns true if there is an open position with the given ticket number */
bool TradeExistTicker(int ticket)
{
    if (ticket <= 0) {
        return (false);
    }
    GetLastError(); /* clear it */
    if (OrderSelect(ticket, SELECT_BY_TICKET)) {
        if (OrderCloseTime() == 0) {
            return (true);    /* an open order */
        } else {
            return (false);    /* historical order */
        }
    } else {
        Print("TradeExistTicker: OrderSelect by ticket failed for: ", ticket, " err=", GetLastError());
        return (true);  /* NOTE RETURN TRUE FOR THIS EA BECAUSE TRUE WILL AVOID MULTIPLE TRADES IN ERR CONDITIONS */
    }
}

int TicketOfExistingTrade(string symbol, int magic)
{
    /* return the ticker of an existing open trade on the given symbol, or -1 if not present.
       The loop through OrderSelect() will pick out only open trades */
    int ticket = -1;

    for (int c = OrdersTotal()-1; c >= 0; c--) {
        if (OrderSelect(c, SELECT_BY_POS, MODE_TRADES) == true) {
            if (OrderSymbol() == symbol && OrderMagicNumber() == magic) {
                if (ticket != -1) {
                    Print("ALERT: MORE THAN ONE OPEN TRADE FOR ", symbol, " FOUND!");
                }
                ticket = OrderTicket();
            }
        }
    }
    return (ticket);
}

double VolAndBalanceAdjustedLotsize(string symbol)
{

    double vol=iATR(symbol, TPSL_ATR_TF, TPSL_ATR_period,0);
    double stop;

    if (LotSizeFromTPnotSL) {
        stop = TP_multiplier*vol;
    } else {
        stop = SL_multiplier*vol;
    }
    return( LibLotsFixedRisk(symbol, stop, AccountBalance() * riskPct / 100.0, depositCurrency ) );
}

//+------------------------------------------------------------------+
//| expert SendOrder function                                        |
//+------------------------------------------------------------------+

double recentSwingLow(string symbol)
{
    /* look for my kind of 'swing low',
       a fractal (local max min with 2 on each side) or max/min of recent 2 M5 bars */
    double f=0.0;
    for (int j=0; j < 50; j++) {
        f = iFractals(symbol,M05,MODE_LOWER,j);
        if (f != 0.0) {
            break;
        }
    }
    f = MathMin(f, iLow(symbol,M05,0));
    f = MathMin(f, iLow(symbol,M05,1));
    return(f);
}

double recentSwingHigh(string symbol)
{
    /* look for my kind of 'swing low',
       a fractal (local max min with 2 on each side) or max/min of recent 2 M5 bars */
    double f=0.0;
    for (int j=0; j < 50; j++) {
        f = iFractals(symbol,M05,MODE_UPPER,j);
        if (f != 0.0) {
            break;
        }
    }
    f = MathMax(f, iHigh(symbol,M05,0));
    f = MathMax(f, iHigh(symbol,M05,1));
    return(f);
}

bool IsSpreadOK(string symbol, int spreadmax)
{
    if (MarketInfo(symbol, MODE_DIGITS) == 3 || MarketInfo(symbol, MODE_DIGITS) == 5) {
        spreadmax *= 10;
    }
    int spread = MarketInfo(symbol,MODE_SPREAD);
    bool ok = (spread <= spreadmax);
    if (ok == false) {
        Print(symbol,"@","Spread=",spread," too high vs max=",spread," (all potentially in subpips)");
    }
    return(ok);
}

int SendOrder(string symbol, int dir)
{
    double price;
    double baseprice;
    double point;
    double sl;
    double tp;

    int ticket = -1;
    int no;

    double volatility=iATR(symbol, TPSL_ATR_TF, TPSL_ATR_period,0);

    //  digit = MarketInfo(symbol, MODE_DIGITS);
    // do we still want this? take += MarketInfo(symbol, MODE_SPREAD) * MarketInfo(symbol, MODE_POINT);

    if (dir == OP_BUY) {
        price = MarketInfo(symbol, MODE_ASK);
        if (SetSLfromSwingExtreme) {
            baseprice = recentSwingLow(symbol);
        } else {
            baseprice=price;
        }
        sl = baseprice - SL_multiplier*volatility; /* OrderReliable will handle the normalization */
        tp = price+TP_multiplier*volatility;
    }

    if (dir == OP_SELL) {
        price = MarketInfo(symbol, MODE_BID);
        if (SetSLfromSwingExtreme) {
            baseprice = recentSwingHigh(symbol);
        } else {
            baseprice=price;
        }

        sl = baseprice + SL_multiplier*volatility;
        tp = price-TP_multiplier*volatility;
    }

    double deltaForLotsize=0.0;
    if (LotSizeFromTPnotSL) {
        deltaForLotsize = tp-price;
    } else {
        deltaForLotsize = price-sl;
    }
    double lots = LibLotsFixedRisk(symbol, deltaForLotsize, AccountBalance() * riskPct / 100.0, depositCurrency );
    no = PosCount(Magic);
    if (no > 1) {
        double newlots = lots / MathPow(2.0, (no-1));
        Print("DEBUG: lowering lots from ", lots, " to ", newlots, " on account of nopen=",no);
        lots = newlots;
    }
    if (lots < MarketInfo(symbol,MODE_MINLOT)) {
        newlots = MarketInfo(symbol,MODE_MINLOT);
        Print("Raising lots from "+lots+" to "+newlots+" on account of min lot size from dealer");
        lots = newlots;
    }
    Print("DEBUG: SendOrder ", symbol, " ",TimeToStr(TimeCurrent()), " price=", price, " sl=",sl, " tp=",tp," lots=",lots);
    ticket = OrderSendReliable(symbol, dir, lots, price, 3, sl, tp, Key, Magic, 0, Green);
    if (ticket > 0) {
        if (OrderSelect(ticket,SELECT_BY_TICKET)) {
            OrderPrint();
        }
    }
    Print("DEBUG: SendOrder ", symbol, " ",TimeToStr(TimeCurrent()), " returning ticket=",ticket);
  
    return (ticket);
}


bool IsCurrencyActive(string sym, int gmtHHMM)
{
    /* returns true if a 3-letter currency is active during the given GMT HHMM */
    if ((sym == "AUD") || (sym == "NZD")) {
        if ((gmtHHMM >= 2200) || (gmtHHMM < 0600)) {
            return (true);
        } else {
            return (false);
        }
    }


    if (sym == "JPY") {
        if ((gmtHHMM >= 0000) && (gmtHHMM < 0800)) {
            return (true);
        } else {
            return (false);
        }
    }


    if ((sym == "CAD") || (sym == "USD")) {
        if ((gmtHHMM >= 1300) && (gmtHHMM < 2100)) {
            return (true);
        } else {
            return (false);
        }
    }

    if ((sym == "EUR") || (sym == "CHF")) {
        if ((gmtHHMM >= 0700) && (gmtHHMM < 1500)) {
            return (true);
        } else {
            return (false);
        }
    }

    if (sym == "GBP") {
        if ((gmtHHMM >= 0800) && (gmtHHMM < 1600)) {
            return (true);
        } else {
            return (false);
        }
    }
}

bool IsPairActive(string pair, int gmthhmm)
{
    return (IsCurrencyActive(StringSubstr(pair, 0, 3), gmthhmm) || IsCurrencyActive(StringSubstr(pair, 3, 3), gmthhmm));
}

//+------------------------------------------------------------------+
//| expert News function -- a contribution by mbkennel               |
//  http://www.forexfactory.com/showthread.php?p=3510269&highlight=newsexist#post3510269
//+------------------------------------------------------------------+
bool NewsExist()
{
    static int minuteSave = -1;
    static bool NewsSave = true;

    /* do not hammer server each tick */
    if (Minute() == minuteSave) {
        return (NewsSave);
    }
    minuteSave = Minute();
    bool News = false;

    for (int d = 0; d < Npairs; d++) {
        int MinutesSincePrevEvent = iCustom(NULL, 0, "Economic News", Pairs[d], true, false, false, true, true, 1, 0);
        int MinutesUntilNextEvent = iCustom(NULL, 0, "Economic News", Pairs[d], true, false, false, true, true, 1, 1);

        if (MinutesUntilNextEvent < 60 || MinutesSincePrevEvent < 15) {
            News = true;
        }
    }

    string xmlFileName = Month() + "-" + Day() + "-" + Year() + "-" + Symbol() + Period() + "-" + "FFCal.xml";
    int handle = FileOpen(xmlFileName, FILE_BIN | FILE_READ | FILE_WRITE);
    if (handle < 0) {
        News = true;
    } else {
        FileClose(handle);
    }
    NewsSave = News;
    return (News);
}

/* utility functions */

/* return lower cased string */
string lc(string s)
{
    string r=s;
    int n=StringLen(s);
    for (int j=0; j < n; j++) {
        int c = StringGetChar(s,j);
        if ((c >= 65) && (c <= 90)) {
            r=StringSetChar(r,j,c+32);
        }
    }
    return(r);
}


//
// OverBoughtOverSold
//
string buySellFlatString(int trend)
{
    if (trend == +1) {
        return("BUY");
    }
    if (trend == -1) {
        return("SELL");
    }
    return("FLAT");
}


bool OverSold(double RSI, double level)
{
    return (RSI < level);
}

bool OverBought(double RSI, double level)
{
    return(RSI > (100-level));
}

bool RSIQ_OK(double RSIQ)
{
    return(RSIQ >= Min_RSI_Quality);
}


//+------------------------------------------------------------------+
//| expert Trend function                                            |
//+------------------------------------------------------------------+

int Trend(string sym, string& trendstr)
{
    if (TrendMethod==1) {
        return(Trend_20_200_4TF(sym,trendstr));
    }
    if (TrendMethod==2) {
        return(Trend_20_200_3TF(sym,trendstr));
    }
    if (TrendMethod==3) {
        return(Trend_3_H1(sym,trendstr));
    }
}


int Trend_20_200_4TF(string sym, string& trendstr)
{

    trendstr = Trend_2_EMA(sym, M15,20,200)+
               Trend_2_EMA(sym, H01,20,200)+
               Trend_2_EMA(sym, H04,20,200)+
               Trend_2_EMA(sym, D01,20,200);

    if (trendstr == "++++") {
        return(+1);
    }
    if (trendstr == "----") {
        return(-1);
    }
    return(0);
}


int Trend_20_200_3TF(string sym, string& trendstr)
{

    trendstr = Trend_2_EMA(sym, M15,20,200)+
               Trend_2_EMA(sym, H01,20,200)+
               Trend_2_EMA(sym, H04,20,200);
    if (trendstr == "+++") {
        return(+1);
    }
    if (trendstr == "---") {
        return(-1);
    }
    return(0);
}


int Trend_3_H1(string sym, string& trendstr)
{
    trendstr = Trend_2_EMA(sym, H01,25,50)+
               Trend_2_EMA(sym, H01,50,100);
    if (trendstr == "++") {
        return(+1);
    }
    if (trendstr == "--") {
        return(-1);
    }
    return(0);
}


string Trend_2_EMA(string symbol, int tf, int ema_small, int ema_large)
{
    string Pair_Trend = "0"; // Flat

    double EMAa = iMA(symbol, tf, ema_small, 0, MODE_EMA, PRICE_CLOSE, 0);
    double EMAb = iMA(symbol, tf, ema_large, 0, MODE_EMA, PRICE_CLOSE, 0);

    if (EMAa < EMAb) {
        Pair_Trend = "-";    // Down
    }
    if (EMAa > EMAb) {
        Pair_Trend = "+";    // Up
    }
    return (Pair_Trend);
}


//+------------------------------------------------------------------+
//| expert LaguerreRSI function |
//+------------------------------------------------------------------+
double LaguerreRSI(string strPair, int intTF, double dblGamma, int intShift)
{
    int intSize = 100;

    double dblRSI, dblUp, dblDown;
    double L[4], LA[4];

    ArrayInitialize( L, 0.0 );

    for(int inx = intSize; inx >= 0; inx--) {

        double Price = iMA(strPair,intTF,1,0,MODE_EMA,PRICE_CLOSE,inx+intShift);

        ArrayCopy(LA, L);
        dblUp = 0.0;
        dblDown = 0.0;

        L[0] = ( 1 - dblGamma ) * Price + ( dblGamma * LA[0] );
        L[1] = ( -dblGamma * L[0] ) + LA[0] + ( dblGamma * LA[1] );
        L[2] = ( -dblGamma * L[1] ) + LA[1] + ( dblGamma * LA[2] );
        L[3] = ( -dblGamma * L[2] ) + LA[2] + ( dblGamma * LA[3] );

        if(L[0] >= L[1]) {
            dblUp = L[0] - L[1];
        } else {
            dblDown = L[1] - L[0];
        }

        if(L[1] >= L[2]) {
            dblUp = dblUp + L[1] - L[2];
        } else {
            dblDown = dblDown + L[2] - L[1];
        }

        if(L[2] >= L[3]) {
            dblUp = dblUp + L[2] - L[3];
        } else {
            dblDown = dblDown + L[3] - L[2];
        }

        if(dblUp + dblDown != 0.0) {
            dblRSI = dblUp / (dblUp + dblDown);
        }
    }

    return( dblRSI );
}

double RsiQuality(string strPair, int intTF, int period, int intShift)
{
    /* return sum(abs(close-open))  / sum (high-low)  * 100.0 */
    double numer=0.0;
    double denom=0.0;

    for (int j=0; j < period+1; j++) {
        double H=iHigh(strPair,intTF, intShift+j);
        double L=iLow(strPair,intTF, intShift+j);
        double O=iOpen(strPair,intTF, intShift+j);
        double C=iClose(strPair,intTF, intShift+j);
        numer += MathAbs(C-O);
        denom += (H-L);
    }

    return( 100.0 * numer / denom );
}

/*
 * Lot size computation library
 */


double LibLotsMidpoint(string pair)
{
    double bid=MarketInfo(pair,MODE_BID);
    double ask=MarketInfo(pair,MODE_ASK);
    if ((bid > 0) && (ask > 0)) {
        return( (bid+ask)*0.5);
    } else {
        return(0.0);
    }
}

double LibLotsExchangeBase(string first, string second)
{
    /* returns the exchange rate of one unit of 'first' in one unit of 'second'
       handles inverted, returns zero if neither works */
    if (first == second) {
        return(1.0);
    }

    double forward=LibLotsMidpoint(first+second);
    if (forward > 0) {
        return(forward);
    }
    double reverse=LibLotsMidpoint(second+first);
    if (reverse > 0.0) {
        return(1.0 / reverse);
    } else {
        return(0.0);
    }
}

double LibLotsExchange(string first, string second)
{
    double rate = LibLotsExchangeBase(first,second);
    if (rate > 0) {
        return(rate);
    } else {
        double top = LibLotsExchangeBase(first,"USD");
        double bot = LibLotsExchangeBase(second,"USD");
        if (bot > 0) {
            return(top / bot);
        } else {
            return(0.0);
        }
    }
}



double LibLotsLotValue(string pair, string deposit)
{
    /* USER CALLABLE FUNCTION

       return the traded value of one lot of 'pair' traded, expressed in
       deposit currency e.g. if pair == "EURUSD" and deposit="USD", and
       lotsize = 100000, then LibLotsLotValue will be EURUSD * 100000

       if pair == "GBPCHF" and deposit currency = "CAD" and lotsize =
       100000 then lotvalue will be GBPCAD */
    return( MarketInfo(pair,MODE_LOTSIZE) * LibLotsExchange( StringSubstr(pair,0,3), deposit));

}

double LibLotsFixedRisk(string pair, double deltaprice, double money, string deposit)
{
    /* USER CALLABLE FUNCTION

       Given a trade in currency 'pair', with entry/stoploss p1,p2
       (which doesn't matteR) return the number of lots necessary so
       that the difference in p1 to p2 yields 'money' in 'deposit'
    */
    deltaprice=MathAbs(deltaprice);

    double riskOneLot = deltaprice * MarketInfo(pair,MODE_LOTSIZE) * LibLotsExchange(StringSubstr(pair,3,3),deposit);
    double lotsraw = money / riskOneLot;
    double Step = MarketInfo(pair, MODE_LOTSTEP);
    int Decimals;

    if (Step == 0.01) {
        Decimals = 2;
    }
    if (Step == 0.10) {
        Decimals = 1;
    }
    if (Step == 1.00) {
        Decimals = 0;
    }

    double rv = NormalizeDouble(lotsraw, Decimals);
    return( rv  );
}

